var Partition = function()
{
	var partition = {};
	var debug = 0;
	
	var selectedSpecimen;
	
	var spWeightsAreValid = [];
	var apWeightsAreValid = [];
	var hisWeightsAreValid = [];
	var nofPiecesAreValid = [];

	var APPEARANCE_OPTIONS = ['', 'HOMOGENEOUS', 'HETEROGENEOUS'];
	var NECROSIS_OPTIONS =	[0, 5, 10, 15, 20, 30, 40, 60, 80, 100];
	var FAT_OPTIONS = [0, 20, 40, 60, 80, 100];
	
	// Page initialization
	partition.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', partition.validateStep1);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', partition.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', partition.validateStep2);
		Events.addEventHandler('partitionDate', 'change', Wizard.validateDate);
		Buttons.addClickHandler('downloadlabels', partition.downloadLabelFile);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', partition.submit);
		
		var url = '../PartitionRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSpecimenTubes';
		Wizard.showLoadingAnimation('Loading specimen tubes...');
		Wizard.asyncJsonRequest(url, partition.specimenTubesLoaded);
	}
	
	partition.specimenTubesLoaded = function(response)
	{
		var frm = document.forms['reggie'];	

		var specimenTubes = response.specimenTubes;
		if (specimenTubes.length > 0)
		{
			for (var i = 0; i < specimenTubes.length; i++)
			{
				var specimen = specimenTubes[i];
				var storeInfo = partition.getStoredInfo(specimen.id);
				var option = new Option(specimen.name + (storeInfo ? ' [*]' : ''), specimen.id, storeInfo ? true : false);
				option.specimenTube = specimen;
				frm.specimentubes[frm.specimentubes.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No specimen available for partitioning');
			return;
		}
		
		Doc.show('step-1');
		Doc.show('gonext');
	}

	partition.validateStep1 = function(event)
	{
		var frm = document.forms['reggie'];	
		for (var i = 0; i < frm.specimentubes.length; i++)
		{
			if (frm.specimentubes[i].selected) 
			{
				Wizard.setInputStatus('specimentubes', 'valid');
				return;
			}
		}

		Wizard.setInputStatus('specimentubes', 'invalid', 'At least one specimen tube must be selected');
		event.preventDefault();
	}

	
	partition.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		selectedSpecimen = [];
		for (var i = 0; i < frm.specimentubes.length; i++)
		{
			if (frm.specimentubes[i].selected) 
			{
				selectedSpecimen[selectedSpecimen.length] = frm.specimentubes[i].specimenTube;
			}
		}
		selectedSpecimen.sort(partition.sortByPosition);
			
		var html = '<table id="weightsTable" class="step-form">';
		html += '<thead>';
		// First header row
		html += '<tr>';
		html += '<th>Specimen</th>';
		html += '<th class="dottedleft">Bloody</th>';
		html += '<th>Appearance</th>';
		html += '<th>Necrosis</th>';
		html += '<th>Fat</th>';
		html += '<th>Coloured</th>';
		html += '<th>NofPieces</th>';
		html += '<th>ApWeight</th>';
		html += '<th>Mult¹</th>';
		html += '<th>HisWeight</th>';
		html += '<th>SpWeight</th>';
		html += '</tr>';
		// Second header row
		html += '<tr>';
		html += '<td>Storage box</td>';
		html += '<td class="dottedleft"></td>';
		html += '<td></td>';
		html += '<td>%</td>';
		html += '<td>%</td>';
		html += '<td></td>';
		html += '<td></td>';
		html += '<td>(mg)</td>';
		html += '<td></td>';
		html += '<td>(mg)</td>';
		html += '<td>(mg)</td>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';

		var appearance = partition.buildOptionsHtml(APPEARANCE_OPTIONS);
		var necrosis = partition.buildOptionsHtml(NECROSIS_OPTIONS);
		var fat = partition.buildOptionsHtml(FAT_OPTIONS);
			
		for (var i = 0; i < selectedSpecimen.length; i++)
		{
			spWeightsAreValid[i] = false;
			apWeightsAreValid[i] = false;
			hisWeightsAreValid[i] = false;
			nofPiecesAreValid[i] = false;
			
			var specimen = selectedSpecimen[i];
			html += '<tr class="highlight">';
			html += '<td class="prompt">'+Strings.encodeTags(specimen.name)+'</td>';
			
			html += '<td class="dottedleft"><input type="checkbox" name="bloody.'+i+'" id="bloody.'+i+'" data-next-focus="appearance.'+i+'" /></td>';
			html += '<td><select name="appearance.'+i+'" id="appearance.'+i+'" data-next-focus="necrosis.'+i+'">'+appearance+'</select></td>';
			html += '<td><select name="necrosis.'+i+'" id="necrosis.'+i+'" data-next-focus="fat.'+i+'">'+necrosis+'</select></td>';	
			html += '<td><select name="fat.'+i+'" id="fat.'+i+'" data-next-focus="coloured.'+i+'">'+fat+'</select></td>';
			html += '<td><input type="checkbox" name="coloured.'+i+'" id="coloured.'+i+'" data-next-focus="nofPieces.'+i+'" /></td>'
			html += '<td><input type="text" class="required" name="nofPieces.'+i+'" id="nofPieces.'+i+'" data-next-focus="apWeight.'+i+'" style="width: 2em;" maxlength="2" /></td>';
			html += '<td><input type="text" class="required" name="apWeight.'+i+'" id="apWeight.'+i+'" data-next-focus="multPieces.'+i+'" style="width: 4em;" maxlength="5" /></td>';			
			html += '<td><input type="checkbox" name="multPieces.'+i+'" id="multPieces.'+i+'" data-next-focus="hisWeight.'+i+'" /></td>'
			html += '<td><input type="text" class="required" name="hisWeight.'+i+'" id="hisWeight.'+i+'" data-next-focus="spWeight.'+i+'" style="width: 4em;" maxlength="5" /></td>';
			html += '<td><input type="text" class="required" name="spWeight.'+i+'" id="spWeight.'+i+'" data-next-focus="bloody.'+(i+1)+'" style="width: 4em;" maxlength="5" /></td>';
			html += '<td class="status" id="specimen.'+i+'.status"></td>';
			html += '</tr>';

			html += '<tr>';
			html += '<td>';
			// Only show box information if there is something show.			
			if (specimen.bioWell)
			{
				html += Strings.encodeTags(specimen.bioWell.bioPlate.name+' ['+specimen.bioWell.location+']');
			}
			html += '</td>';
			html += '<td colspan="11" class="help dottedleft"><span class="message" id="specimen.'+i+'.message"></span><span id="specimen.'+i+'.help">&nbsp;</span></td>';
			html += '</tr>';
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('weightInputs').innerHTML = html;
		
		// Attach event handlers, restore from local storage, etc.
		isRestoringFromLocalStorage = true;
		for (var i = 0; i < selectedSpecimen.length; i++)
		{
			var elements = ['bloody.'+i, 'appearance.'+i, 'necrosis.'+i, 'fat.'+i, 'coloured.'+i, 'nofPieces.'+i, 'apWeight.'+i, 'multPieces.'+i, 'hisWeight.'+i, 'spWeight.'+i];
			partition.addEventHandlers(elements, 'focus', partition.onFocusElement);
			partition.addEventHandlers(elements, 'blur', partition.onFocusElement);
			partition.addEventHandlers(elements, 'keypress', Wizard.focusOnEnter);
			partition.addEventHandlers(elements, 'change', partition.saveToLocalStorage);
			
			Events.addEventHandler('nofPieces.'+i, 'keypress', Events.integerOnly);
			Events.addEventHandler('nofPieces.'+i, 'change', partition.nofPiecesOnChange);
			Events.addEventHandler('apWeight.'+i, 'keypress', Events.numberOnly);
			Events.addEventHandler('apWeight.'+i, 'change', partition.apWeightOnChange);
			Events.addEventHandler('hisWeight.'+i, 'keypress', Events.numberOnly);
			Events.addEventHandler('hisWeight.'+i, 'change', partition.hisWeightOnChange);
			Events.addEventHandler('spWeight.'+i, 'keypress', Events.numberOnly);
			Events.addEventHandler('spWeight.'+i, 'change', partition.spWeightOnChange);

			var specimen = selectedSpecimen[i];
			var storeInfo = partition.getStoredInfo(specimen.id);
			if (storeInfo)
			{
				frm['bloody.'+i].checked = storeInfo.bloody;
				Forms.selectListOption(frm['appearance.'+i], storeInfo.appearance);
				Forms.selectListOption(frm['necrosis.'+i], storeInfo.necrosis);
				Forms.selectListOption(frm['fat.'+i], storeInfo.fat);
				frm['coloured.'+i].checked = storeInfo.coloured;
				frm['nofPieces.'+i].value = storeInfo.nofPieces;
				frm['spWeight.'+i].value = storeInfo.spWeight;
				frm['hisWeight.'+i].value = storeInfo.hisWeight;
				frm['apWeight.'+i].value = storeInfo.apWeight;
				frm['multPieces.'+i].checked = storeInfo.multPieces;
			
				Events.sendChangeEvent('spWeight.'+i);
				Events.sendChangeEvent('hisWeight.'+i);
				Events.sendChangeEvent('apWeight.'+i);
				Events.sendChangeEvent('nofPieces.'+i);
			}
		}
		
		isRestoringFromLocalStorage = false;
		
		Wizard.setCurrentStep(2);
		frm.partitionDate.focus();
		frm.partitionDate.select();
	
		Doc.show('goregister');
		Doc.show('gocancel');
		Wizard.keepSessionAlive();
	}
	
	partition.addEventHandlers = function(elements, eventName, handler)
	{
		for (var i = 0; i < elements.length; i++)
		{
			Events.addEventHandler(elements[i], eventName, handler);
		}
	}
	
	partition.sortByPosition = function(t1, t2)
	{
		var well1 = t1.bioWell;
		var box1 = well1.bioPlate.name;
		var well2 = t2.bioWell;
		var box2 = well2.bioPlate.name;
		if (box1 == box2)
		{
			return well1.row == well2.row ? well1.column - well2.column : well1.row - well2.row;
		}
		else
		{
			return box1 < box2 ? -1 : 1;
		}
	}

	partition.buildOptionsHtml = function(a)
	{
		var html = '';
		for (var i = 0; i < a.length; i++)
		{
			html += '<option value="'+a[i]+'">'+a[i]+'</option>';
		}
		return html;
	}
	
	partition.downloadLabelFile = function()
	{	
		var url = '../PartitionRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLabelFile';
		
		var tubes = [];
		var frm = document.forms['reggie'];
		for (var i = 0; i < selectedSpecimen.length; i++)
		{
			tubes[tubes.length] = selectedSpecimen[i].name;
		}
		url += '&tubes='+tubes.join(',');
		window.open(url);
	}

	partition.onFocusElement = function(event)
	{
		var target = event.currentTarget;
		var dotI = target.id.indexOf('.');
		var col = target.id.substr(0, dotI);
		var index = target.id.substr(dotI+1);
		
		var msg = '&nbsp;';
		if (event.type == 'focus')
		{
			if (col == 'bloody')
			{
				msg = 'Check this if the partition is bloody. Unchecked is default.';
			}
			else if (col == 'appearance')
			{
				msg = 'Select appearance. Empty is default.';
			}
			else if (col == 'necrosis')
			{
				msg = 'Select percentage of necrosis. \'0\' is default.';
			}
			else if (col == 'fat')
			{
				msg = 'Select percentage of fat. \'0\' is default.';
			}
			else if (col == 'coloured')
			{
				msg = 'Check this if the partition is coloured. Unchecked is default.';
			}
			else if (col == 'nofPieces')
			{
				msg = 'Number of pieces in this partition.';
			}
			else if (col == 'apWeight')
			{
				msg = 'Ap weight in [mg].';
			}
			else if (col == 'hisWeight')
			{
				msg = 'His weight in [mg].';
			}
			else if (col == 'spWeight')
			{
				msg = 'Sp weight in [mg].';
			}
			else if (col == 'multPieces')
			{
				msg = 'Check the box if multiple specimen pieces are sampled for the AllPrep piece.';
			}
			// So we get an error icon for empty required fields
			if (target.value == '') Events.sendChangeEvent(event.target);
		}
		Doc.element('specimen.'+index+'.help').innerHTML = msg;
	}

	partition.nofPiecesOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var dotI = target.id.indexOf('.');
		var index = parseInt(target.id.substr(dotI+1));
		
		var nofPieces = parseInt(target.value);
		if (nofPieces > 0)
		{
			nofPiecesAreValid[index] = true;
			if (nofPieces == 1)
			{
				frm['multPieces.'+index].disabled = true;
				frm['multPieces.'+index].checked = false;
			}
			else
			{
				frm['multPieces.'+index].disabled = false;
			}
		}
		else
		{
			nofPiecesAreValid[index] = false;
		}
		partition.updateInputStatus(index);
	}
	
	partition.apWeightOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var dotI = target.id.indexOf('.');
		var index = parseInt(target.id.substr(dotI+1));

		var apWeight = parseFloat(target.value);
		apWeightsAreValid[index] = apWeight >= 0;
		partition.updateInputStatus(index);
	}

	partition.spWeightOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var dotI = target.id.indexOf('.');
		var index = parseInt(target.id.substr(dotI+1));

		var spWeight = parseFloat(target.value);
		spWeightsAreValid[index] = spWeight >= 0;
		partition.updateInputStatus(index);
	}
	
	partition.hisWeightOnChange = function(event)
	{
		var frm = document.forms['reggie'];
		var target = event.currentTarget;
		var dotI = target.id.indexOf('.');
		var index = parseInt(target.id.substr(dotI+1));

		var hisWeight = parseFloat(target.value);
		hisWeightsAreValid[index] = hisWeight >= 0;
		partition.updateInputStatus(index);
	}
	
	partition.updateInputStatus = function(index)
	{
		var frm = document.forms['reggie'];
		
		if (!nofPiecesAreValid[index])
		{
			Wizard.setInputStatus('specimen.'+index, 'invalid', 'NofPieces is invalid.');
		}
		else if (!apWeightsAreValid[index])
		{
			Wizard.setInputStatus('specimen.'+index, 'invalid', 'APWeight is invalid.');
		}
		else if (!hisWeightsAreValid[index])
		{
			Wizard.setInputStatus('specimen.'+index, 'invalid', 'HisWeight is invalid.');
		}
		else if (!spWeightsAreValid[index])
		{
			Wizard.setInputStatus('specimen.'+index, 'invalid', 'SpWeight is invalid.');
		}
		else
		{
			var spWeightValue = parseFloat(frm.elements['spWeight.'+index].value);
			var apWeightValue = parseFloat(frm.elements['apWeight.'+index].value);
			var hisWeightValue = parseFloat(frm.elements['hisWeight.'+index].value);
			if (apWeightValue == 0)
			{
				Wizard.setInputStatus('specimen.'+index, 'warning', 'No Lysate item will be created!');
			}
			else if (hisWeightValue == 0)
			{
				Wizard.setInputStatus('specimen.'+index, 'warning', 'No Histology item will be created!');
			}
			else 
			{
				Wizard.setInputStatus('specimen.'+index, 'valid');
			}
		}
	}

	partition.validateStep2 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('partitionDate');
		
		if (valid)
		{
			for (var i = 0; i < selectedSpecimen.length; i++)
			{
				valid &= spWeightsAreValid[i];
				valid &= apWeightsAreValid[i];
				valid &= hisWeightsAreValid[i];
				valid &= nofPiecesAreValid[i];
				if (!valid) 
				{
					Wizard.setInputStatus('specimen.'+i, 'invalid');
					break;
				}
			}
		}
		
		if (!valid) event.preventDefault();
	}

	
	partition.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		submitInfo.partitionDate = frm.partitionDate.value;
		
		var specimens = [];
		for (var i = 0; i < selectedSpecimen.length; i++)
		{
			var specimen = selectedSpecimen[i];
			
			specimen.spWeight = parseFloat(frm['spWeight.'+i].value) || null;
			specimen.apWeight = parseFloat(frm['apWeight.'+i].value) || null;
			specimen.hisWeight = parseFloat(frm['hisWeight.'+i].value) || null;
			specimen.nofPieces = parseInt(frm['nofPieces.'+i].value);

			specimen.multPieces = frm['multPieces.'+i].checked;
			
			// Generate partition comment string
			var comment = [];	
			if (frm['bloody.'+i].checked) comment[comment.length] = 'BLOODY';
			if (frm['appearance.'+i].selectedIndex > 0) comment[comment.length] = frm['appearance.'+i].value;
			if (frm['necrosis.'+i].selectedIndex > 0) comment[comment.length] = 'NECROSIS'+frm['necrosis.'+i].value+'%';
			if (frm['fat.'+i].selectedIndex > 0) comment[comment.length] = 'FAT'+frm['fat.'+i].value+'%';
			if (frm['coloured.'+i].checked) comment[comment.length] = 'COLOURED';
			specimen.operatorPartitionComment = comment.length > 0 ? comment.join(';') : null;

			specimens[i] = specimen;		
		}
		submitInfo.specimens = specimens;
		
		var url = '../PartitionRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=CreatePartitions';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, partition.submissionResults, 'POST', JSON.stringify(submitInfo));
		
	}
	
	partition.submissionResults = function(response)
	{
		for (var i = 0; i < response.partitioned.length; i++)
		{
			partition.removeFromLocalStorage(response.partitioned[i]);
		}
		
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}


	/**
		Save as much information as possible about the partitioned
		specimen tube. The information is stored in the localStorage()
		under the key 'reggie.specimen.<id>.partitionInfo' where
		<id> is the ID of the specimen tube.
	*/
	partition.saveToLocalStorage = function(event)
	{
		if (!isRestoringFromLocalStorage)
		{
			var target = event.currentTarget;
			var dotI = target.id.indexOf('.');
			var index = parseInt(target.id.substr(dotI+1));
			
			var specimen = selectedSpecimen[index];
			var frm = document.forms['reggie'];
			
			var info = {};
			info.id = specimen.id;
			info.bloody = frm['bloody.'+index].checked;
			info.appearance = frm['appearance.'+index].value;
			info.necrosis = frm['necrosis.'+index].value;
			info.fat = frm['fat.'+index].value;
			info.coloured = frm['coloured.'+index].checked;
			info.nofPieces = frm['nofPieces.'+index].value;
			info.spWeight = frm['spWeight.'+index].value;
			info.hisWeight = frm['hisWeight.'+index].value;
			info.apWeight = frm['apWeight.'+index].value;
			info.multPieces = frm['multPieces.'+index].checked;
			App.setLocal('reggie.specimen.' + specimen.id + '.partitionInfo', JSON.stringify(info));
			if (debug)
			{
				App.debug(index + ': ' + App.getLocal('reggie.specimen.' + specimen.id + '.partitionInfo'));
			}
		}
	}

	partition.removeFromLocalStorage = function(specimenId)
	{
		App.removeLocal('reggie.specimen.' + specimenId + '.partitionInfo');
	}
	
	partition.getStoredInfo = function(specimenId)
	{
		try
		{
			var stored = App.getLocal('reggie.specimen.' + specimenId + '.partitionInfo');
			if (debug) App.debug(stored);
			return stored ? JSON.parse(stored) : null;
		}
		catch (e)
		{
			App.debug('Could not load stored information for tube ' + specimenId + ': ' + e);
		}
	}

	
	return partition;
}();

Doc.onLoad(Partition.initPage);

